#!/bin/bash

# Author: Victor Correal (victor.correal@bsc.es)
# Date  : 13/10/2023 

source ../bash_utils/helpers.sh

usage() {
    info "Usage: $0 --model-dir=path_to_bundle_folder"
    info "OPTIONAL FLAGS:"
    info "[--nodes=<nodes>] [--queue=<queue>] [--MPI=<val>] [--OpenMP=<val>] [--timelimit=<time>]"
    info "[--flength=d5|h20] [--disable-io] [--outdir=<outdir>] [--io-tasks-ifs=<tasks>]"
    info "[--io-tasks-nemo=<tasks>] [--script=name of script under SLURM/<machine>]"
    info "[--outdir=<output_dir>] [--indir=<input_dir>]"
    info "[--force] [--help]"
    info "Available resolutions: tco79-eORCA1 tco399-eORCA025 tco1279-eORCA12"
}

parse_args() {
    OPTIONS=$(getopt -o "" --long model-dir:,script:,resolution:,nodes:,queue:,MPI:,OpenMP:,timelimit:,disable-io,flength:,outdir:,indir:,io-tasks-nemo:,io-tasks-ifs:,force,help -n "$0" -- "$@")
    eval set -- "$OPTIONS"

    while true; do
        case "$1" in
            --model-dir)
                MODEL_DIR="$2"
                shift 2
            ;;
            --script)
                SCRIPT_NAME="$2"
                shift 2
            ;;
            --resolution)
                RESOLUTION="$2"
                shift 2
            ;;
            --io-tasks-ifs)
                IO_TASKS_IFS="$2"
                shift 2
            ;;
            --io-tasks-nemo)
                IO_TASKS_NEMO="$2"
                shift 2
            ;;
            --disable-io)
                IO_ENABLED=false
                IO_TASKS_IFS=0
                IO_TASKS_NEMO=0
                shift
            ;;
            --force)
                FORCE_RUN=true
                shift
            ;;
            --flength)
                F_LENGHT="$2"
                shift 2
            ;;
            --nodes)
                NODES="$2"
                shift 2
            ;;
            --queue)
                QUEUE="$2"
                shift 2
            ;;
            --MPI)
                MPI_PER_NODE="$2"
                shift 2
            ;;
            --OpenMP)
                OMP_PER_TASK="$2"
                shift 2
            ;;
            --timelimit)
                TIME_LIMIT="$2"
                shift 2
            ;;
            --help)
                usage
                exit 0
            ;;
            --outdir)
                OUTDIR="$2"
                shift 2
            ;;
            --indir)
                INDIR="$2"
                shift 2
            ;;
            --help)
                usage
                exit 0
            ;;
            --)
                shift
                break
            ;;
            *)
                usage
                exit 1
            ;;
        esac
    done
}

print_options() {
    info "=============================INSTALLATION==========================================="
    info "MODEL: $MODEL_DIR"
    info "RAPS: $RAPS_DIR"
    info "SUBMISSION SCRIPT: $RAPS_DIR/bin/SLURM/mn5/$SCRIPT_NAME"
    info "=============================ALLOCATION RESOURCES==================================="
    info "QUEUE: $QUEUE, TIMELIMIT: $TIME_LIMIT"
    info "NODE COUNT: $NODES, MPI PER NODE: $MPI_PER_NODE, OPENMP PER TASK: $OMP_PER_TASK"
    info "================================JOB OPTIONS========================================="
    info "RESOULTION: $RESOLUTION -- FORECAST LENGHT : $F_LENGHT -- IO_ENABLED? : $IO_ENABLED"
    info "IO TASKS IFS: $IO_TASKS_IFS -- IO TASKS NEMO : $IO_TASKS_NEMO"
    info "FLAGS: $OTHER_VAR"
    info "===================================================================================="
    info "INPUT DIR: $INDIR"  
    info "OUT DIR: $OUTDIR"
    info "===================================================================================="

    if [ $FORCE_RUN = false ]; then
        echo "¿FINE? (Y/N)"
        read ANSW
        case "$ANSW" in
            [Yy])
                
                ;;
            *)
                fatal "USER INPUT $ANSW"
                ;;
        esac
    else
        echo "Force run ENABLE"
    fi
}

check_required_args() {
    if [ -z "${MODEL_DIR}" ] || [ -z "${RESOLUTION}" ]; then
        usage
        fatal "MODEL_DIR, RESOLUTION are required."
    fi
}

check_dirs(){
    [ -d $MODEL_DIR ] || fatal "$MODEL_DIR not found"
    
    BDIR=${MODEL_DIR}/build
    RAPS_DIR=${MODEL_DIR}/source/raps
    SCRIPT_PATH=$RAPS_DIR/bin/SLURM/$RAPS_HOST_NAME/$SCRIPT_NAME

    [ -d $BDIR ] || fatal "build dir not found at $BDIR"
    [ -d $RAPS_DIR ] || fatal "raps dir not found at $RAPS_DIR"
    
}

check_submision_script(){
    if [ ! -f $SCRIPT_PATH ]; then
        info "Submission script $SCRIPT_PATH not found"
        info "Available:"
        ls -la $RAPS_DIR/bin/SLURM/$RAPS_HOST_NAME/*.slurm
        fatal "unable to find submission script "
    fi
}

check_slurm_options(){
    if [ "$NODES" != "DEFAULT" ]; then
        SLURM_SYN_NODES="-N $NODES"
        echo "Overiding default for NODES: $SLURM_SYN_NODES"
    fi

    if [ "$QUEUE" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_QUEUE="-q $QUEUE"
        echo "Overiding default for QUEUE: $SLURM_SYNTAXIS_QUEUE"
    fi

    if [ "$TIME_LIMIT" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_TIME_LIMIT="--time $TIME_LIMIT"
        echo "Overiding default for TIME_LIMIT: $SLURM_SYNTAXIS_TIME_LIMIT"
    fi

    if [ "$MPI_PER_NODE" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_MPI_PER_NODE="--ntasks-per-node $MPI_PER_NODE"
        echo "Overiding default for MPI_PER_NODE: $SLURM_SYNTAXIS_MPI_PER_NODE"
    fi

    if [ "$OMP_PER_TASK" != "DEFAULT" ]; then
        SLURM_SYNTAXIS_OMP_PER_TASK="--cpus-per-task $OMP_PER_TASK"
        echo "Overiding default for OMP_PER_TASK: $SLURM_SYNTAXIS_OMP_PER_TASK"
    fi
}
